// MinRabbit v1.1
// By TruSoulja - trusoulja@zbzoom.net
// http://www.isaboo.net/trusoulja

// DisplayName = MinRabbit

//--- GAME RULES BEGIN ---
//MinRabbit v1.1 by TruSoulja
//  No ELF, Laser Rifle, Chaingun, Missile,
//    Mortar, Concussion or Whiteouts.
//  Do not leave the duel area as rabbit
//    or suffer damage.
//  Get points by midairing opponents.
//    Longer shots are worth more points.
//--- GAME RULES END ---

$DeathMessageOOB[0] = '\c0%1 was killed for running from the duel area.';

package MinRabbitGame {
  function Flag::objectiveInit(%data, %flag) {
    $flagStatus = "<At Home>";
    %flag.carrier = "";
    %flag.originalPosition = %flag.getTransform();
    %flag.isHome = true;
    %flag.hide(false);

    // create a waypoint to the flag's starting place
    %flagWaypoint = new WayPoint() {
      position = %flag.position;
      rotation = "1 0 0 0";
      name = "Flag Home";
      dataBlock = "WayPointMarker";
      team = $NonRabbitTeam;
    };

    $AIRabbitFlag = %flag;

    MissionCleanup.add(%flagWaypoint);
  }

  function Armor::damageObject(%data, %targetObject, %sourceObject, %position, %amount, %damageType, %momVec, %mineSC) {
    Game.testForMA(%targetObject, %sourceObject, %amount, %damageType);

    parent::damageObject(%data, %targetObject, %sourceObject, %position, %amount, %damageType, %momVec, %mineSC);
  }
};

$InvBanList[MinRabbit, "TurretOutdoorDeployable"] = 1;
$InvBanList[MinRabbit, "TurretIndoorDeployable"] = 1;
$InvBanList[MinRabbit, "ElfBarrelPack"] = 1;
$InvBanList[MinRabbit, "MortarBarrelPack"] = 1;
$InvBanList[MinRabbit, "PlasmaBarrelPack"] = 1;
$InvBanList[MinRabbit, "AABarrelPack"] = 1;
$InvBanList[MinRabbit, "MissileBarrelPack"] = 1;
$InvBanList[MinRabbit, "MissileLauncher"] = 1;
$InvBanList[MinRabbit, "Mortar"] = 1;
$InvBanList[MinRabbit, "Chaingun"] = 1;
$InvBanList[MinRabbit, "SniperRifle"] = 1;
$InvBanList[MinRabbit, "ELFGun"] = 1;
$InvBanList[MinRabbit, "FlashGrenade"] = 1;
$InvBanList[MinRabbit, "ConcussionGrenade"] = 1;
$InvBanList[MinRabbit, "Mine"] = 1;

function MinRabbitGame::init(%game) {
  %game.missionArea = %game.getObjectByName(MissionGroup, "MissionArea");
  %game.defMissionArea = %game.missionArea.area;

  // temp
  %game.soundsOn = 1;
  %game.alertMAClients = 1;

  %game.missionAreaRadius = 100;

  // minimums
  %game.minMAHeight = 2;
  %game.minFlagHeight = 5;
  %game.minDiscDamage = 0.49;
  %game.minPlasmaDamage = 0.449;
  %game.minSLDamage = 0.449;
  %game.minSLBackDamage = 1.349;
  %game.minGrenadeDamage = 0.37;

  // rabbit modifier
  %game.rabbitMod = 2;

  // base points
  %game.pDisc = 1;
  %game.pPlasma = 1;
  %game.pSL = 1;
  %game.pSLBack = 3;
  %game.pGrenade = 10;

  // points per meter
  %game.ppmDisc = 0.135;
  %game.ppmPlasma = 0.22;
  %game.ppmSL = 0.388;
  %game.ppmSLBack = 0.388;
  %game.ppmGrenade = 0.14;
}

function MinRabbitGame::setUpTeams(%game) {
  // Force the numTeams variable to one:
  DefaultGame::setUpTeams(%game);
  %game.numTeams = 1;
  setSensorGroupCount(3);

  //team damage should always be off for Rabbit
  $teamDamage = 0;

  //make all the sensor groups visible at all times
  if (!Game.teamMode) {
    setSensorGroupAlwaysVisMask($NonRabbitTeam, 0xffffffff);
    setSensorGroupAlwaysVisMask($RabbitTeam, 0xffffffff);

    // non-rabbits can listen to the rabbit: all others can only listen to self
    setSensorGroupListenMask($NonRabbitTeam, (1 << $RabbitTeam) | (1 << $NonRabbitTeam));
  }
}

function MinRabbitGame::initGameVars(%game) {
  %game.init();

  %game.playerBonusValue = 1;
  %game.playerBonusTime = 3 * 1000; //3 seconds

  %game.teamBonusValue = 1;
  %game.teamBonusTime = 15 * 1000; //15 seconds
  %game.flagReturnTime = 30 * 1000;  //45 seconds

  %game.waypointFrequency = 24000;
  %game.waypointDuration = 6000;
}

$RabbitTeam = 2;
$NonRabbitTeam = 1;

function MinRabbitGame::allowsProtectedStatics(%game) {
  return true;
}

function MinRabbitGame::clientMissionDropReady(%game, %client) {
  messageClient(%client, 'MsgClientReady', "", "RabbitGame");
  messageClient(%client, 'MsgYourScoreIs', "", 0);
  //messageClient(%client, 'MsgYourRankIs', "", -1);
  messageClient(%client, 'MsgRabbitFlagStatus', "", $flagStatus);

  messageClient(%client, 'MsgMissionDropInfo', '\c0You are in mission %1 (%2).', $MissionDisplayName, $MissionTypeDisplayName, $ServerName );

  DefaultGame::clientMissionDropReady(%game, %client);

  // set their score to 0
  %game.resetScore(%client);
}

function MinRabbitGame::AIHasJoined(%game, %client) {
  //let everyone know the player has joined the game
  //messageAllExcept(%client, -1, 'MsgClientJoinTeam', '%1 has joined the hunt.', %client.name, "", %client, $NonRabbitTeam);
}

function MinRabbitGame::clientJoinTeam( %game, %client, %team, %respawn ) {
  %game.assignClientTeam( %client );

  // Spawn the player:
  %game.spawnPlayer( %client, %respawn );
}


function MinRabbitGame::assignClientTeam(%game, %client) {
  // all players start on team 1
  %client.team = $NonRabbitTeam;

  // set player's skin pref here
  setTargetSkin(%client.target, %client.skin);

  // Let everybody know you are no longer an observer:
  messageAll( 'MsgClientJoinTeam', '\c1%1 has joined the hunt.', %client.name, "", %client, %client.team );
  updateCanListenState( %client );
}

function MinRabbitGame::playerSpawned(%game, %player) {
  //call the default stuff first...
  DefaultGame::playerSpawned(%game, %player);

  //find the rabbit
  %clRabbit = -1;
  for (%i = 0; %i < ClientGroup.getCount(); %i++) {
    %cl = ClientGroup.getObject(%i);
    if (isObject(%cl.player) && isObject(%cl.player.holdingFlag)) {
       %clRabbit = %cl;
       break;
    }
  }

  //now set a waypoint just for that client...
  cancel(%player.client.waypointSchedule);
  if (isObject(%clRabbit) && !%player.client.isAIControlled())
    %player.client.waypointSchedule = %game.showRabbitWaypointClient(%clRabbit, %player.client);

  %player.client.outOfBounds = false;

  buyFavorites(%player.client);
  %game.fixFavorites(%player);
  %player.selectWeaponSlot(0);
}

function MinRabbitGame::pickPlayerSpawn(%game, %client, %respawn) {
  // all spawns come from team 1
  return %game.pickTeamSpawn($NonRabbitTeam);
}

function MinRabbitGame::createPlayer(%game, %client, %spawnLoc, %respawn) {
  %client.team = $NonRabbitTeam;
  DefaultGame::createPlayer(%game, %client, %spawnLoc, %respawn);
}

function MinRabbitGame::recalcScore(%game, %client) {
  //score is grabs + kills + style + (totalTime / 7.5 seconds);
  %timeHoldingFlagMS = %client.flagTimeMS;
  if (isObject(%client.player.holdingFlag))
    %timeHoldingFlagMS += getSimTime() - %client.startTime;
  %client.score = %client.flagGrabs + %client.kills + %client.mrStyle + mFloor(%timeHoldingFlagMS / 7500);
  messageClient(%client, 'MsgYourScoreIs', "", %client.score);
  %game.recalcTeamRanks(%client);
  %game.checkScoreLimit(%client);
}

function MinRabbitGame::onClientDamaged(%game, %clVictim, %clAttacker, %damageType, %sourceObject) {
  //if the victim is the rabbit, and the attacker is not the rabbit, set the damage time...
  if (isObject(%clAttacker) && %clAttacker != %clVictim) {
    if (%clVictim.team == $RabbitTeam)
       %game.rabbitDamageTime = getSimTime();
  }

  //call the default
  DefaultGame::onClientDamaged(%game, %clVictim, %clAttacker, %damageType, %sourceObject);
}

function MinRabbitGame::onClientKilled(%game, %clVictim, %clKiller, %damageType, %implement, %damageLoc) {
  %plVictim = %clVictim.player;
  if (%plVictim.getState() !$= "Dead")
    bottomPrintAll("Error occurred.", 15);

  if (%clVictim.outOfBounds) {
    %game.skipOOBMessage = true;
    %game.enterMissionArea(%plVictim.getDatablock(), %plVictim);
  }

  //see if the killer was the rabbit and the victim was someone else...
  if (isObject(%clKiller) && (%clKiller != %clVictim) && (%clKiller.team == $RabbitTeam))
  {
    %clKiller.kills++;
    %game.recalcScore(%clKiller);
  }

  DefaultGame::onClientKilled(%game, %clVictim, %clKiller, %damageType, %implement, %damageLoc);
}

function MinRabbitGame::playerDroppedFlag(%game, %player) {
  //set the flag status
  %flag = %player.holdingFlag;
  %player.holdingFlag = "";
  %flag.carrier = "";
  $flagStatus = "<In the Field>";

  %player.unmountImage($FlagSlot);
  %flag.hide(false);

  //hide the rabbit waypoint
  cancel(%game.waypointSchedule);
  %game.hideRabbitWaypoint(%player.client);

  //set the client status
  %player.client.flagTimeMS += getSimTime() - %player.client.startTime;
  %player.client.team = $NonRabbitTeam;
  %player.client.setSensorGroup($NonRabbitTeam);
  setTargetSensorGroup(%player.getTarget(), $NonRabbitTeam);

  messageAllExcept(%player.client, -1, 'MsgRabbitFlagDropped', '\c2%1 dropped the flag!~wfx/misc/flag_drop.wav', %player.client.name);
  // if the player left the mission area, he's already been notified
  if(!%player.outArea)
    messageClient(%player.client, 'MsgRabbitFlagDropped', '\c2You dropped the flag!~wfx/misc/flag_drop.wav');
  logEcho(%player.client.nameBase@" (pl "@%player@"/cl "@%player.client@") dropped flag");

  %flag.returnThread = %game.schedule(%game.flagReturnTime, "returnFlag", %flag);

  // clear waypoints
  %game.clearDuelAreaWaypoints();
}

function MinRabbitGame::playerTouchFlag(%game, %player, %flag) {
  if ((%flag.carrier $= "") && (%player.getState() !$= "Dead")) {
    %player.client.startTime = getSimTime();
    %player.holdingFlag = %flag;
    %flag.carrier = %player;
    %player.mountImage(FlagImage, $FlagSlot, true); //, $teamSkin[$RabbitTeam]);
    cancel(%flag.returnThread);
    %flag.hide(true);
    %flag.isHome = false;
    $flagStatus = %client.name;
    messageAll('MsgRabbitFlagTaken', '\c2%1 has taken the flag!~wfx/misc/flag_snatch.wav', %player.client.name);
    logEcho(%player.client.nameBase@" (pl "@%player@"/cl "@%player.client@") took flag");
    %player.client.team = $RabbitTeam;
    %player.client.setSensorGroup($RabbitTeam);
    setTargetSensorGroup(%player.getTarget(), $RabbitTeam);

    //increase the score
    %player.client.flagGrabs++;
    %game.recalcScore(%player.client);
    %game.schedule(5000, "RabbitFlagCheck", %player);

    //show the rabbit waypoint
    %game.rabbitDamageTime = 0;
    cancel(%game.waypointSchedule);
    %game.showRabbitWaypoint(%player.client);

    // change missionarea
    %game.changeMissionArea(%player);
  }
}

function MinRabbitGame::rabbitFlagCheck(%game, %player) {
  // this function calculates the score for the rabbit. It must be done periodically
  // since the rabbit's score is based on how long the flag has been in possession.
  if((%player.holdingFlag != 0) && (%player.getState() !$= "Dead"))
  {
    %game.recalcScore(%player.client);
    //reschedule this flagcheck for 2.5 seconds
    %game.schedule(5000, "RabbitFlagCheck", %player);
  }
}

function MinRabbitGame::returnFlag(%game, %flag) {
  messageAll('MsgRabbitFlagReturned', '\c2The flag was returned to its starting point.~wfx/misc/flag_return.wav');
  logEcho("flag return (timeout)");
  %game.resetFlag(%flag);
}

function MinRabbitGame::resetFlag(%game, %flag) {
  %flag.setVelocity("0 0 0");
  %flag.setTransform(%flag.originalPosition);
  %flag.isHome = true;
  %flag.carrier = "";
  $flagStatus = "<At Home>";
  %flag.hide(false);
}

// ----- These functions are native to Rabbit

function MinRabbitGame::timeLimitReached(%game) {
  logEcho("game over (timelimit)");
  %game.gameOver();
  cycleMissions();
}

function MinRabbitGame::scoreLimitReached(%game) {
  logEcho("game over (scorelimit)");
  %game.gameOver();
  cycleMissions();
}

function MinRabbitGame::checkScoreLimit(%game, %client) {
  %scoreLimit = MissionGroup.Rabbit_scoreLimit;
  // default of 1200 if scoreLimit not defined  (that's 1200 seconds worth - 20 minutes)
  if(%scoreLimit $= "")
    %scoreLimit = 1200;
  if(%client.score >= %scoreLimit)
    %game.scoreLimitReached();
}

function MinRabbitGame::gameOver(%game) {
  //call the default
  DefaultGame::gameOver(%game);

  //send the message
  messageAll('MsgGameOver', "Match has ended.~wvoice/announcer/ann.gameover.wav" );

  cancel(%game.rabbitWaypointThread);
  messageAll('MsgClearObjHud', "");
  for(%i = 0; %i < ClientGroup.getCount(); %i++)
  {
    %client = ClientGroup.getObject(%i);
    %game.resetScore(%client);
    cancel(%client.waypointSchedule);
  }

  %game.clearDuelAreaWaypoints();

  cancel(%game.waypointSchedule);
  cancel(%game.removeNamesSchedule);
}

function MinRabbitGame::resetScore(%game, %client) {
  %client.score = 0;
  %client.mrStyle = 0;
  %client.kills = 0;
  %client.deaths = 0;
  %client.suicides = 0;
  %client.flagGrabs = 0;
  %client.flagTimeMS = 0;
}

function MinRabbitGame::enterMissionArea(%game, %playerData, %player) {
  if (%player.client.outOfBounds && %player.client.team == $RabbitTeam) {
    %player.client.outOfBounds = false;
    if (!%game.skipOOBMessage)
      messageClient(%player.client, 'EnterMissionArea', '\c1You are back in the rabbit duel area.');
    %game.skipOOBMessage = false;
    logEcho(%player.client.nameBase@" (pl "@%player@"/cl "@%player.client@") entered mission area");
    cancel(%player.alertThread);
  }
}

function MinRabbitGame::leaveMissionArea(%game, %playerData, %player) {
  if(%player.getState() $= "Dead")
    return;

  if (!%player.client.outOfBounds && %player.client.team == $RabbitTeam) {
    %player.client.outOfBounds = true;
    messageClient(%player.client, 'LeaveMissionArea', '\c1You have left the rabbit duel area. Return or take damage.~wfx/misc/warning_beep.wav');
    logEcho(%player.client.nameBase@" (pl "@%player@"/cl "@%player.client@") left mission area");
    cancel(%player.alertThread);
    %player.alertThread = %game.schedule(1000, "AlertPlayer", 3, %player);
  }
}

function MinRabbitGame::AlertPlayer(%game, %count, %player) {
  if (%player.client.team == $RabbitTeam) {
    if(%count > 1)
       %player.alertThread = %game.schedule(1000, "AlertPlayer", %count - 1, %player);
    else
       %player.alertThread = %game.schedule(1000, "MissionAreaDamage", %player);
  }

  //keep the thread going for non rabbits, but give the new rabbit time to return to the mission area...
  else
    %player.alertThread = %game.schedule(1000, "AlertPlayer", 3, %player);
}

function MinRabbitGame::MissionAreaDamage(%game, %player) {
  if(%player.getState() !$= "Dead") {
    %player.setDamageFlash(0.1);
  	%damageRate = 0.1;
  	%pack = %player.getMountedImage($BackpackSlot);
  	if(%pack.getName() $= "RepairPackImage")
  		if(%player.getMountedImage($WeaponSlot).getName() $= "RepairGunImage")
  			%damageRate = 0.15;
    %prevHurt = %player.getDamageLevel();
    %player.setDamageLevel(%prevHurt + %damageRate);
    // a little redundancy to see if the lastest damage killed the player
    if(%player.getState() $= "Dead")
       %game.onClientKilled(%player.client, 0, $DamageType::OutOfBounds);
    else
       %player.alertThread = %game.schedule(1000, "MissionAreaDamage", %player);
  } else {
    %game.onClientKilled(%player.client, 0, $DamageType::OutOfBounds);
  }
}

function MinRabbitGame::dropFlag(%game, %player) {
  //you can no longer throw the flag in Rabbit...
}

function MinRabbitGame::updateScoreHud(%game, %client, %tag) {
  //tricky stuff here...  use two columns if we have more than 15 clients...
  %numClients = $TeamRank[0, count];
  if (%numClients > $ScoreHudMaxVisible)
    %numColumns = 2;

  // Clear the header:
  messageClient(%client, 'SetScoreHudHeader', "", "");

  // Send subheader:
  if (%numColumns == 2)
    messageClient(%client, 'SetScoreHudSubheader', "", '<tab:5,155,225,305,455,525>\tPLAYER\tSCORE\tTIME\tPLAYER\tSCORE\tTIME');
  else
    messageClient(%client, 'SetScoreHudSubheader', "", '<tab:15,235,335>\tPLAYER\tSCORE\tTIME');

  //recalc the score for whoever is holding the flag
  if (isObject($AIRabbitFlag.carrier))
    %game.recalcScore($AIRabbitFlag.carrier.client);

  %countMax = %numClients;
  if (%countMax > (2 * $ScoreHudMaxVisible)) {
    if (%countMax & 1)
      %countMax++;
    %countMax = %countMax / 2;
  } else if (%countMax > $ScoreHudMaxVisible)
    %countMax = $ScoreHudMaxVisible;

  for (%index = 0; %index < %countMax; %index++) {
    //get the client info
    %col1Client = $TeamRank[0, %index];
    %col1ClientScore = %col1Client.score $= "" ? 0 : %col1Client.score;
    %col1Style = "";

    if (isObject(%col1Client.player.holdingFlag)) {
      %col1ClientTimeMS = %col1Client.flagTimeMS + getSimTime() - %col1Client.startTime;
      %col1Style = "<color:00dc00>";
    } else {
      %col1ClientTimeMS = %col1Client.flagTimeMS;
      if (%col1Client == %client)
        %col1Style = "<color:dcdcdc>";
    }

    if (%col1ClientTimeMS <= 0)
      %col1ClientTime = "";
    else {
      %minutes = mFloor(%col1ClientTimeMS / (60 * 1000));
      if (%minutes <= 0)
        %minutes = "0";
      %seconds = mFloor(%col1ClientTimeMS / 1000) % 60;
      if (%seconds < 10)
        %seconds = "0" @ %seconds;

      %col1ClientTime = %minutes @ ":" @ %seconds;
    }

    //see if we have two columns
    if (%numColumns == 2) {
      %col2Client = "";
      %col2ClientScore = "";
      %col2ClientTime = "";
      %col2Style = "";

      //get the column 2 client info
      %col2Index = %index + %countMax;
      if (%col2Index < %numClients) {
        %col2Client = $TeamRank[0, %col2Index];
        %col2ClientScore = %col2Client.score $= "" ? 0 : %col2Client.score;

        if (isObject(%col2Client.player.holdingFlag)) {
          %col2ClientTimeMS = %col2Client.flagTimeMS + getSimTime() - %col2Client.startTime;
          %col2Style = "<color:00dc00>";
        } else {
          %col2ClientTimeMS = %col2Client.flagTimeMS;
          if ( %col2Client == %client )
            %col2Style = "<color:dcdcdc>";
        }

        if (%col2ClientTimeMS <= 0)
          %col2ClientTime = "";
        else {
          %minutes = mFloor(%col2ClientTimeMS / (60 * 1000));
          if (%minutes <= 0)
            %minutes = "0";
          %seconds = mFloor(%col2ClientTimeMS / 1000) % 60;
          if (%seconds < 10)
            %seconds = "0" @ %seconds;

          %col2ClientTime = %minutes @ ":" @ %seconds;
        }
      }
    }

    //if the client is not an observer, send the message
    if (%client.team != 0)
      if (%numColumns == 2)
        messageClient(%client, 'SetLineHud', "", %tag, %index, '<tab:10><spush>%7\t<clip:150>%1</clip><rmargin:205><just:right>%2<rmargin:270><just:right>%3<spop><rmargin:505><lmargin:310>%8<just:left>%4<rmargin:505><just:right>%5<rmargin:570><just:right>%6',
                      %col1Client.name, %col1ClientScore, %col1ClientTime, %col2Client.name, %col2ClientScore, %col2ClientTime, %col1Style, %col2Style);
      else
        messageClient(%client, 'SetLineHud', "", %tag, %index, '<tab:20>%4\t<clip:200>%1</clip><rmargin:280><just:right>%2<rmargin:375><just:right>%3',
                      %col1Client.name, %col1ClientScore, %col1ClientTime, %col1Style);
    else
      if (%numColumns == 2)
        if (%col2Style $= "<color:00dc00>")
          messageClient(%client, 'SetLineHud', "", %tag, %index, '<tab:10><spush>%7\t<clip:150><a:gamelink\t%8>%1</a></clip><rmargin:205><just:right>%2<rmargin:270><just:right>%3<spop><rmargin:505><lmargin:310><color:00dc00><just:left><clip:150><a:gamelink\t%9>%4</a></clip><rmargin:505><just:right>%5<rmargin:570><just:right>%6',
                        %col1Client.name, %col1ClientScore, %col1ClientTime,
                        %col2Client.name, %col2ClientScore, %col2ClientTime,
                        %col1Style, %col1Client, %col2Client);
        else if (%col2Style $= "<color:dcdcdc>")
          messageClient(%client, 'SetLineHud', "", %tag, %index, '<tab:10><spush>%7\t<clip:150><a:gamelink\t%8>%1</a></clip><rmargin:205><just:right>%2<rmargin:270><just:right>%3<spop><rmargin:505><lmargin:310><color:dcdcdc><just:left><clip:150><a:gamelink\t%9>%4</a></clip><rmargin:505><just:right>%5<rmargin:570><just:right>%6',
                        %col1Client.name, %col1ClientScore, %col1ClientTime,
                        %col2Client.name, %col2ClientScore, %col2ClientTime,
                        %col1Style, %col1Client, %col2Client);
        else
          messageClient(%client, 'SetLineHud', "", %tag, %index, '<tab:10><spush>%7\t<clip:150><a:gamelink\t%8>%1</a></clip><rmargin:205><just:right>%2<rmargin:270><just:right>%3<spop><rmargin:505><lmargin:310><just:left><clip:150><a:gamelink\t%9>%4</a></clip><rmargin:505><just:right>%5<rmargin:570><just:right>%6',
                        %col1Client.name, %col1ClientScore, %col1ClientTime,
                        %col2Client.name, %col2ClientScore, %col2ClientTime,
                        %col1Style, %col1Client, %col2Client);
      else
        messageClient(%client, 'SetLineHud', "", %tag, %index, '<tab:20>%4\t<clip:200><a:gamelink\t%5>%1</a></clip><rmargin:280><just:right>%2<rmargin:375><just:right>%3',
                      %col1Client.name, %col1ClientScore, %col1ClientTime, %col1Style, %col1Client);
  }

  // Tack on the list of observers:
  %observerCount = 0;
  for (%i = 0; %i < ClientGroup.getCount(); %i++) {
    %cl = ClientGroup.getObject(%i);
    if (%cl.team == 0)
      %observerCount++;
  }

  if (%observerCount > 0) {
    messageClient(%client, 'SetLineHud', "", %tag, %index, "");
    %index++;
    messageClient(%client, 'SetLineHud', "", %tag, %index, '<tab:10, 310><spush><font:Univers Condensed:22>\tOBSERVERS (%1)<rmargin:260><just:right>TIME<spop>', %observerCount);
    %index++;
    for (%i = 0; %i < ClientGroup.getCount(); %i++) {
      %cl = ClientGroup.getObject(%i);
      //if this is an observer
      if (%cl.team == 0) {
        %obsTime = getSimTime() - %cl.observerStartTime;
        %obsTimeStr = %game.formatTime(%obsTime, false);
        messageClient(%client, 'SetLineHud', "", %tag, %index, '<tab:20, 310>\t<clip:150>%1</clip><rmargin:260><just:right>%2',
                      %cl.name, %obsTimeStr);
        %index++;
      }
    }
  }

  //clear the rest of Hud so we don't get old lines hanging around...
  messageClient( %client, 'ClearHud', "", %tag, %index );
}

function MinRabbitGame::showRabbitWaypointClient(%game, %clRabbit, %client) {
  //make sure we have a rabbit
  if (!isObject(%clRabbit) || !isObject(%clRabbit.player) || !isObject(%clRabbit.player.holdingFlag))
    return;

  //no waypoints for bots
  if (%client.isAIControlled())
    return;

  //scope the client, then set the always vis mask...
  %clRabbit.player.scopeToClient(%client);
  %visMask = getSensorGroupAlwaysVisMask(%clRabbit.getSensorGroup());
  %visMask |= (1 << %client.getSensorGroup());
  setSensorGroupAlwaysVisMask(%clRabbit.getSensorGroup(), %visMask);

  //now issue a command to kill the target
  %client.setTargetId(%clRabbit.target);
  commandToClient(%client, 'TaskInfo', %client, -1, false, "Kill the Rabbit!");
  %client.sendTargetTo(%client, true);

  //send the "waypoint is here sound"
  messageClient(%client, 'MsgRabbitWaypoint', '~wfx/misc/target_waypoint.wav');

  //and hide the waypoint
  %client.waypointSchedule = %game.schedule(%game.waypointDuration, "hideRabbitWaypointClient", %clRabbit, %client);
}

function MinRabbitGame::hideRabbitWaypointClient(%game, %clRabbit, %client) {
  //no waypoints for bots
  if (%client.isAIControlled())
    return;

  //unset the always vis mask...
  %visMask = getSensorGroupAlwaysVisMask(%clRabbit.getSensorGroup());
  %visMask &= ~(1 << %client.getSensorGroup());
  setSensorGroupAlwaysVisMask(%clRabbit.getSensorGroup(), %visMask);

  //kill the actually task...
  removeClientTargetType(%client, "AssignedTask");
}

function MinRabbitGame::showRabbitWaypoint(%game, %clRabbit) {
  //make sure we have a rabbit
  if (!isObject(%clRabbit) || !isObject(%clRabbit.player) || !isObject(%clRabbit.player.holdingFlag))
    return;

  //only show the rabbit waypoint if the rabbit hasn't been damaged within the frequency period
  if (getSimTime() - %game.rabbitDamageTime < %game.waypointFrequency) {
    %game.waypointSchedule = %game.schedule(%game.waypointFrequency, "showRabbitWaypoint", %clRabbit);
    return;
  }

  //loop through all the clients and flash a waypoint at the rabbits position
  for (%i = 0; %i < ClientGroup.getCount(); %i++) {
    %cl = ClientGroup.getObject(%i);
    if (%cl.isAIControlled() || %cl == %clRabbit)
       continue;

    //scope the client, then set the always vis mask...
    %clRabbit.player.scopeToClient(%cl);
    %visMask = getSensorGroupAlwaysVisMask(%clRabbit.getSensorGroup());
    %visMask |= (1 << %cl.getSensorGroup());
    setSensorGroupAlwaysVisMask(%clRabbit.getSensorGroup(), %visMask);

    //now issue a command to kill the target
    %cl.setTargetId(%clRabbit.target);
    commandToClient(%cl, 'TaskInfo', %cl, -1, false, "Kill the Rabbit!");
    %cl.sendTargetTo(%cl, true);

    //send the "waypoint is here sound"
    messageClient(%cl, 'MsgRabbitWaypoint', '~wfx/misc/target_waypoint.wav');
  }

  //schedule the time to hide the waypoint
  %game.waypointSchedule = %game.schedule(%game.waypointDuration, "hideRabbitWaypoint", %clRabbit);
}

function MinRabbitGame::hideRabbitWaypoint(%game, %clRabbit) {
  //make sure we have a valid client
  if (!isObject(%clRabbit))
    return;

  //loop through all the clients and hide the waypoint
  for (%i = 0; %i < ClientGroup.getCount(); %i++) {
    %cl = ClientGroup.getObject(%i);
    if (%cl.isAIControlled())
      continue;

    //unset the always vis mask...
    %visMask = getSensorGroupAlwaysVisMask(%clRabbit.getSensorGroup());
    %visMask &= ~(1 << %cl.getSensorGroup());
    setSensorGroupAlwaysVisMask(%clRabbit.getSensorGroup(), %visMask);

    //kill the actually task...
    removeClientTargetType(%cl, "AssignedTask");
  }

  //make sure we have a rabbit before scheduling the next showRabbitWaypoint...
  if (isObject(%clRabbit.player) && isObject(%clRabbit.player.holdingFlag))
    %game.waypointSchedule = %game.schedule(%game.waypointFrequency, "showRabbitWaypoint", %clRabbit);
}

function MinRabbitGame::updateKillScores(%game, %clVictim, %clKiller, %damageType, %implement) {
  if (%game.testTurretKill(%implement))   //check for turretkill before awarded a non client points for a kill
    %game.awardScoreTurretKill(%clVictim, %implement);
  else if (%game.testKill(%clVictim, %clKiller)) { //verify victim was an enemy
    %game.awardScoreKill(%clKiller);
    %game.awardScoreDeath(%clVictim);
  } else {
   if (%game.testSuicide(%clVictim, %clKiller, %damageType))  //otherwise test for suicide
     %game.awardScoreSuicide(%clVictim);
   else
     if (%game.testTeamKill(%clVictim, %clKiller)) //otherwise test for a teamkill
       %game.awardScoreTeamKill(%clVictim, %clKiller);
  }
}

function MinRabbitGame::applyConcussion(%game, %player) {
  // MES -- this won't do anything, the function MinRabbitGame::dropFlag is empty
  %game.dropFlag( %player );
}

// ------------ misc functions

function MinRabbitGame::getObjectDistance(%game, %obj1, %obj2) {
  return VectorDist(%obj1.getTransform(), %obj2.getTransform());
}

function MinRabbitGame::calcMAScore(%game, %player, %damageType, %maDistance, %amount) {
  %mod = 1;
  if (isObject(%player.holdingFlag))
    %mod = %game.rabbitMod;

  switch (%damageType) {
    case 2:
      return %mod * (%game.pPlasma + mfloor(%game.ppmPlasma * %maDistance + 0.5));
    case 4:
      return %mod * (%game.pDisc + mfloor(%game.ppmDisc * %maDistance + 0.5));
    case 5:
      return %mod * (%game.pGrenade + mfloor(%game.ppmGrenade * %maDistance + 0.5));
    case 10:
      if (%amount > 0.5)
        return %mod * (%game.pSLBack + mfloor(%game.ppmSLBack * %maDistance + 0.5));
      else
        return %mod * (%game.pSL + mfloor(%game.ppmSL * %maDistance + 0.5));
    default:
      error("Unknown damage type in calcMAScore().");
      return 0;
  }
}

function MinRabbitGame::areEnemies(%game, %team1, %team2) {
  if (%team1 == "1" && %team2 == "2")
    return true;
  else if (%team1 == "2" && %team2 == "1")
    return true;

  return false;
}

function MinRabbitGame::getObjectHeight(%game, %object) {
  %x = getWord(%object.getTransform(), 0);
  %y = getWord(%object.getTransform(), 1);
  %z = getWord(%object.getTransform(), 2) + 10;
  %posFrom = %x SPC %y SPC %z;
  return %game.getPositionHeight(%posFrom) - 10;
}

function MinRabbitGame::getPositionHeight(%game, %pos) {
  %posTo = VectorAdd(%pos, "0 0 -10000");
  %hit1 = getWords(containerRayCast(%pos, %posTo, $TypeMasks::TerrainObjectType, 0), 1, 3);
  return VectorDist(%pos, %hit1);
}

function MinRabbitGame::testForMA(%game, %targetObject, %sourceObject, %amount, %damageType) {
  if (!%game.areEnemies(%targetObject.client.team, %sourceObject.client.team))
    return;

  if (%game.isMidair(%game.getObjectHeight(%targetObject), %amount, %damageType)) {
    %maDistance = %game.getObjectDistance(%targetObject, %sourceObject);
    %points = %game.calcMAScore(%sourceObject, %damageType, %maDistance, %amount);
    %sourceObject.client.mrStyle += %points;
    %game.recalcScore(%sourceObject.client);

    // round to two spots
    %maDistance = %maDistance * 10.0;
    %maDistance = mfloor(%maDistance);
    %maDistance = %maDistance / 10.0;

    if (%damageType == 10 && %amount > 0.5)
      %damageText = "shocklance in the back";
    else
      %damageText = $DamageTypeText[%damageType];

    %clientMsg = "";
    %othersMsg = "";
    if (%game.alertMAClients) {
      %clientMsg = "\c5You scored " @ %points @ " points for a " @ %maDistance @ "m " @ %damageText @ " midair.";
      %othersMsg = "\c2" @ %sourceObject.client.nameBase @ " scored " @ %points @ " points for a " @ %maDistance @ "m " @ %damageText @ " midair.";
    }

    %clientSound = "";
    %othersSound = "";
    if (%game.soundsOn) {
      %clientSound = "~wfx/misc/flag_lost.wav";
      %othersSound = "~wfx/vehicles/crash_ground_vehicle.wav";
    }

    if (%game.soundsOn || %game.alertMAClients) {
      messageClient(%sourceObject.client, 'MinRabbitMA', %clientMsg @ %clientSound);
      messageAllExcept(%sourceObject.client, -1, 'MinRabbitMADet', %othersMsg @ %othersSound);
    }
  }
}

function MinRabbitGame::isMidair(%game, %objHeight, %amount, %damageType) {
  if (%objHeight < %game.minMAHeight)
    return false;

  switch (%damageType) {
    case 2:
      if (%amount >= %game.minPlasmaDamage)
        return true;
    case 4:
      if (%amount >= %game.minDiscDamage)
        return true;
    case 5:
      if (%amount >= %game.minGrenadeDamage)
        return true;
    case 10:
      if (%amount >= %game.minSLDamage)
        return true;
  }

  return false;
}

function MinRabbitGame::changeMissionArea(%game, %player) {
  %transform = %player.getTransform();
  %centerX = getWord(%transform, 0);
  %centerY = getWord(%transform, 1);

  %game.missionAreaTop = %centerY - %game.missionAreaRadius;
  %game.missionAreaLeft = %centerX - %game.missionAreaRadius;

  %game.missionArea.setArea(%game.missionAreaLeft, %game.missionAreaTop, %game.missionAreaRadius * 2, %game.missionAreaRadius * 2);

  %game.setDuelAreaWaypoints(%game.missionAreaTop, %game.missionArealeft, "Duel Area Corner", 1);
}

function MinRabbitGame::setDuelAreaWaypoints(%game, %top, %left, %name, %reschedule) {
  // top left waypoint
  %point1Pos = %left SPC %top SPC 0;
  %point1Pos = %left SPC %top SPC getTerrainHeight(%point1Pos);
  $mrBorderWaypoint1 = new WayPoint() {
    position = %point1Pos;
    rotation = "1 0 0 0";
    name = %name;
    dataBlock = "WayPointMarker";
    team = $RabbitTeam;
  };

  // top right waypoint
  %point2Pos = %left + %game.missionAreaRadius * 2 SPC %top SPC 0;
  %point2Pos = %left + %game.missionAreaRadius * 2 SPC %top SPC getTerrainHeight(%point2Pos);
  $mrBorderWaypoint2 = new WayPoint() {
    position = %point2Pos;
    rotation = "1 0 0 0";
    name = %name;
    dataBlock = "WayPointMarker";
    team = $RabbitTeam;
  };

  // bottom left waypoint
  %point3Pos = %left SPC %top + %game.missionAreaRadius * 2 SPC 0;
  %point3Pos = %left SPC %top + %game.missionAreaRadius * 2 SPC getTerrainHeight(%point3Pos);
  $mrBorderWaypoint3 = new WayPoint() {
    position = %point3Pos;
    rotation = "1 0 0 0";
    name = %name;
    dataBlock = "WayPointMarker";
    team = $RabbitTeam;
  };

  // bottom right waypoint
  %point4Pos = %left + %game.missionAreaRadius * 2 SPC %top + %game.missionAreaRadius * 2 SPC 0;
  %point4Pos = %left + %game.missionAreaRadius * 2 SPC %top + %game.missionAreaRadius * 2 SPC getTerrainHeight(%point4Pos);
  $mrBorderWaypoint4 = new WayPoint() {
    position = %point4Pos;
    rotation = "1 0 0 0";
    name = %name;
    dataBlock = "WayPointMarker";
    team = $RabbitTeam;
  };

  if (%reschedule) {
    cancel(%game.removeNamesSchedule);
    %game.removeNamesSchedule = %game.schedule(3000, "removeWaypointNames");
  }
}

function MinRabbitGame::removeWaypointNames(%game) {
  %game.clearDuelAreaWaypoints();
  %game.setDuelAreaWaypoints(%game.missionAreaTop, %game.missionAreaLeft, "", 0);
}

function MinRabbitGame::clearDuelAreaWaypoints(%game) {
  cancel(%game.removeNamesSchedule);
  $mrBorderWaypoint1.delete();
  $mrBorderWaypoint2.delete();
  $mrBorderWaypoint3.delete();
  $mrBorderWaypoint4.delete();
}

function MinRabbitGame::getObjectByName(%game, %group, %name) {
  %count = %group.getCount();
  for (%i = 0; %i < %count; %i++) {
    %groupObject = %group.getObject(%i);
    %groupName = strlwr(%groupObject.getName());
    if (%groupName $= strLwr(%name))
      return %groupObject;
  }
  return "";
}

function MinRabbitGame::fixFavorites(%game, %player) {
  %player.setInventory("TurretOutdoorDeployable", 0);
  %player.setInventory("TurretIndoorDeployable", 0);
  %player.setInventory("ElfBarrelPack", 0);
  %player.setInventory("MortarBarrelPack", 0);
  %player.setInventory("PlasmaBarrelPack", 0);
  %player.setInventory("AABarrelPack", 0);
  %player.setInventory("MissileBarrelPack", 0);
  %player.setInventory("MissileLauncher", 0);
  %player.setInventory("Mortar", 0);
  %player.setInventory("Chaingun", 0);
  %player.setInventory("SniperRifle", 0);
  %player.setInventory("ELFGun", 0);
  %player.setInventory("FlashGrenade", 0);
  %player.setInventory("ConcussionGrenade", 0);
  %player.setInventory("Mine", 0);
}


